# Version 0.1
#
#  TeachingSchedule is copyright 2005 Jeffrey G. Pittman.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#  MA 02111-1307 USA

from Products.Archetypes.public import BaseFolderSchema, Schema
from Products.Archetypes.public import StringField, IntegerField, FloatField, LinesField, BooleanField
from Products.Archetypes.public import LinesWidget, TextAreaWidget, IdWidget, IntegerWidget, DecimalWidget, StringWidget, BooleanWidget, MultiSelectionWidget
from Products.Archetypes.public import BaseFolder, registerType
from Products.TeachingSchedule.config import PROJECTNAME
from Products.CMFCore import CMFCorePermissions

import sys,string,cStringIO

from config import *

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

from reportlab.pdfgen.canvas import Canvas

try :
    from Shared.reportlab.platypus.flowables import Image
    from Shared.reportlab.platypus.paragraph import Paragraph
    from Shared.reportlab.platypus.doctemplate import *
    from Shared.reportlab.lib.units import inch
    from Shared.reportlab.lib.units import cm
    from Shared.reportlab.lib import styles
    from Shared.PIL import Image as PIL_Image
    from reportlab.lib import colors as rlcolors
    from reportlab.graphics.shapes import *
    from reportlab.graphics.charts.textlabels import Label
    from reportlab.platypus import BaseDocTemplate, PageTemplate, Frame, Paragraph
    from reportlab.lib.styles import ParagraphStyle
    from reportlab.lib.pagesizes import A4
except ImportError :
    from reportlab.platypus import BaseDocTemplate, PageTemplate, Frame, Paragraph
    from reportlab.lib.styles import ParagraphStyle
    from reportlab.lib.pagesizes import A4
    from reportlab.platypus.flowables import Image
    from reportlab.platypus.paragraph import Paragraph
    from reportlab.platypus.doctemplate import *
    from reportlab.lib.units import inch
    from reportlab.lib.units import cm
    from reportlab.lib import styles
    from PIL import Image as PIL_Image
    from reportlab.lib import colors as rlcolors
    from reportlab.graphics.shapes import *
    from reportlab.graphics.charts.textlabels import Label

from Products.TeachingSchedule.drawing import TeachingScheduleDrawing
from Products.TeachingSchedule.sample_data import sample_courses

drawing_base_schema = BaseFolderSchema +  Schema((
     StringField('id',
                 required=0, ## Still actually required, but
                             ## the widget will supply the missing value
                             ## on non-submits
                 mode="rw",
                 accessor="getId",
                 mutator="setId",
                 default=None,
                 widget=IdWidget(label="Short Name",
                                 description="Should not contain spaces, underscores or mixed case. "\
                                 "Short Name is part of the item's web address.",
                                 visible={'view' : 'visible'}),
                 ),
    StringField('title',
                required=1,
                searchable=1,
                default='',
                accessor='Title',
                widget=StringWidget(label="Title",
                                    description="This name shows in the navigation menu."),
                ),
    StringField('description',
                isMetadata=1,
                accessor='Description',
                widget=TextAreaWidget(label='Description', 
                              description='This description is optional, but not a bad idea for search text purposes.'),
                ),
    StringField('render_types',
                vocabulary=RENDER_TYPES,
                mutator="setRenderTypes",
                accessor="RenderTypes",
                widget=MultiSelectionWidget(label='Render Types',
                                            description='Select render types to generate.',
                                            format='checkbox'),
                ),
    IntegerField('left_margin',
                 required=1,
                 default='70',
                 validators=("isInt"),
                 accessor='LeftMargin',
                 widget=IntegerWidget(label="LeftMargin",
                                      description="Left Margin of Image, in pixels."),
                 ),
    IntegerField('right_margin',
                 required=1,
                 default='70',
                 validators=("isInt"),
                 accessor='RightMargin',
                 widget=IntegerWidget(label="RightMargin",
                                      description="Right Margin of Image, in pixels."),
                 ),
    IntegerField('top_margin',
                 required=1,
                 default='20',
                 validators=("isInt"),
                 accessor='TopMargin',
                 widget=IntegerWidget(label="TopMargin",
                                      description="Top Margin, in pixels."),
                 ),
    IntegerField('bottom_margin',
                 required=1,
                 default='20',
                 validators=("isInt"),
                 accessor='BottomMargin',
                 widget=IntegerWidget(label="BottomMargin",
                                      description="Bottom Margin, in pixels."),
                 ),
    BooleanField('scale_to_fit',
                  searchable = 0,
                  accessor='isScaleToFit',
                  widget=BooleanWidget(label='Scale to Fit',
                                       description='If yes, will be scaled to fit in width x height size; if no, will be cropped on right and bottom, if necessary.'),
                  ),
                 ))

teaching_schedule_schema = Schema((
    IntegerField('start_year',
                 required=1,
                 default='2005',
                 validators=("isInt"),
                 accessor='StartYear',
                 widget=IntegerWidget(label="StartYear",
                                      description="Start year for entire plot."),
                 ),
    IntegerField('start_month',
                 required=1,
                 default='8',
                 validators=("isInt"),
                 accessor='StartMonth',
                 widget=IntegerWidget(label="StartMonth",
                                      description="Start month for entire plot."),
                 ),
    IntegerField('start_day',
                 required=1,
                 default='24',
                 validators=("isInt"),
                 accessor='StartDay',
                 widget=IntegerWidget(label="StartDay",
                                      description="Start day for entire plot."),
                 ),
    IntegerField('end_year',
                 required=1,
                 default='2005',
                 validators=("isInt"),
                 accessor='EndYear',
                 widget=IntegerWidget(label="EndYear",
                                      description="End year for entire plot."),
                 ),
    IntegerField('end_month',
                 required=1,
                 default='12',
                 validators=("isInt"),
                 accessor='EndMonth',
                 widget=IntegerWidget(label="EndMonth",
                                      description="End month for entire plot."),
                 ),
    IntegerField('end_day',
                 required=1,
                 default='15',
                 validators=("isInt"),
                 accessor='EndDay',
                 widget=IntegerWidget(label="EndDay",
                                      description="End day for entire plot."),
                 ),
    IntegerField('day_width',
                 required=1,
                 default='15',
                 validators=("isInt"),
                 accessor='DayWidth',
                 widget=IntegerWidget(label="DayWidth",
                                      description="Width of day column."),
                 ),
    IntegerField('hour_height',
                 required=1,
                 default='30',
                 validators=("isInt"),
                 accessor='HourHeight',
                 widget=IntegerWidget(label="HourHeight",
                                      description="Height of each hour row."),
                 ),
    IntegerField('bottom_hour',
                 required=1,
                 default='6',
                 validators=("isInt"),
                 accessor='BottomHour',
                 widget=IntegerWidget(label="BottomHour",
                                      description="Hour to start plot on bottom."),
                 ),
    IntegerField('top_hour',
                 required=1,
                 default='21',
                 validators=("isInt"),
                 accessor='TopHour',
                 widget=IntegerWidget(label="TopHour",
                                      description="Hour to end plot on top."),
                 ),
    LinesField('courses_field',
               required=0,
               default='\n'.join(sample_courses),
               accessor="Courses",
               validators=('validate_courses',),
               widget=LinesWidget(label='Courses',
                                  description='For each course, provide: username, year, month, day, hour, minute, duration_minutes, recurrence_interval, start_year, start_month, start_day, end_year, end_month, end_day, course_name, red, green, blue',),
               ),
        ))


class TeachingSchedule(BaseFolder):
    """
    A folder object to hold PNG, SVG, PDF and other images
    generated for a drawing type.
    """
    allowed_content_types = ('Document', 'File',)

    filter_content_types = 1

    global_allow = 1

    schema = drawing_base_schema + teaching_schedule_schema

    content_icon = 'teachingscheduleicon.gif'

    security = ClassSecurityInfo()

    actions = ({'id': 'view',
                'name': 'View',
                'action': 'string:${object_url}/TeachingSchedule_view',
                'permissions': (CMFCorePermissions.View,)},
               {'id': 'references',
                'name': 'References',
                'action': 'string:${object_url}/reference_edit',
                'permissions': (CMFCorePermissions.ModifyPortalContent,),
                'visible':0},
               {'id': 'metadata',
                'name': 'Properties',
                'action': 'string:${object_url}/base_metadata',
                'permissions': (CMFCorePermissions.ModifyPortalContent,),
                'visible':0})

    def gatherInput(self):
        render_types = self.RenderTypes()
        left_margin = self.LeftMargin()
        right_margin = self.RightMargin()
        top_margin = self.TopMargin()
        bottom_margin = self.BottomMargin()
        scale_to_fit = self.isScaleToFit()
        start_year = self.StartYear()
        start_month = self.StartMonth()
        start_day = self.StartDay()
        end_year = self.EndYear()
        end_month = self.EndMonth()
        end_day = self.EndDay()
        day_width = self.DayWidth()
        hour_height = self.HourHeight()
        bottom_hour = self.BottomHour()
        top_hour = self.TopHour()
        courses_input = self.Courses()

        return (render_types, 
                left_margin,
                right_margin,
                top_margin,
                bottom_margin,
                scale_to_fit, 
                start_year,
                start_month,
                start_day,
                end_year,
                end_month,
                end_day,
                day_width,
                hour_height,
                bottom_hour,
                top_hour,
                courses_input)
            
    # processForm is called for creation
    def processForm(self, data=1, metadata=0, REQUEST=None, values=None):
        ret = BaseFolder.processForm(self, data, metadata, REQUEST, values)

        self.createDrawing(self.gatherInput())

        return ret

    # update is called for edit
    def update(self, **kwargs):
        ret = BaseFolder.update(self, **kwargs)

        self.createDrawing(self.gatherInput())

        return ret

    def createDrawing(self, input_items):
        render_types_with_messages, left_margin, right_margin, top_margin, bottom_margin, scale_to_fit, start_year, start_month, start_day, end_year, end_month, end_day, day_width, hour_height, bottom_hour, top_hour, courses_input = input_items

        # trim off the parenthetic messages in the render_types
        render_types = []
        for image_type in render_types_with_messages:
            if len(image_type) > 0: # extra one returned
                render_types.append(image_type.split()[0])

        teaching_schedule = TeachingScheduleDrawing(render_types, 
                                                    left_margin,
                                                    right_margin,
                                                    top_margin,
                                                    bottom_margin,
                                                    scale_to_fit, 
                                                    start_year,
                                                    start_month,
                                                    start_day,
                                                    end_year,
                                                    end_month,
                                                    end_day,
                                                    day_width,
                                                    hour_height,
                                                    bottom_hour,
                                                    top_hour,
                                                    courses_input)
    
        drawing = teaching_schedule.getDrawing()

        # save the drawing in the selected formats
        if 'PNG' in render_types:
            existing_file = getattr(self, 'drawing.png', None)
            if existing_file:
                 self.manage_delObjects('drawing.png')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='PNG')
            self.invokeFactory('File', id='drawing.png')
            png_file = self['drawing.png']
            png_file.update_data(data=render_result)
    
        if 'JPG' in render_types:
            existing_file = getattr(self, 'drawing.jpg', None)
            if existing_file:
                 self.manage_delObjects('drawing.jpg')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='JPG')
            self.invokeFactory('File', id='drawing.jpg')
            jpg_file = self['drawing.jpg']
            jpg_file.update_data(data=render_result)

        if 'GIF' in render_types:
            existing_file = getattr(self, 'drawing.gif', None)
            if existing_file:
                 self.manage_delObjects('drawing.gif')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='GIF')
            self.invokeFactory('File', id='drawing.gif')
            gif_file = self['drawing.gif']
            gif_file.update_data(data=render_result)
    
        if 'TIFF' in render_types:
            existing_file = getattr(self, 'drawing.tiff', None)
            if existing_file:
                 self.manage_delObjects('drawing.tiff')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='TIFF')
            self.invokeFactory('File', id='drawing.tiff')
            tiff_file = self['drawing.tiff']
            tiff_file.update_data(data=render_result)
    
        if 'PCT' in render_types:
            existing_file = getattr(self, 'drawing.pct', None)
            if existing_file:
                 self.manage_delObjects('drawing.pct')

            from reportlab.graphics import renderPM
            render_result = renderPM.drawToString(drawing, fmt='PCT')
            self.invokeFactory('File', id='drawing.pct')
            pct_file = self['drawing.pct']
            pct_file.update_data(data=render_result)
    
        if 'SVG-via-ReportLab' in render_types:
            existing_file = getattr(self, 'drawing-via-ReportLab.svg', None)
            if existing_file:
                 self.manage_delObjects('drawing-via-ReportLab.svg')

            from reportlab.graphics import renderSVG
            render_result = renderSVG.drawToString(drawing)
            self.invokeFactory('File', id='drawing-via-ReportLab.svg')
            svg_file = self['drawing-via-ReportLab.svg']
            svg_file.update_data(data=render_result)

        if 'SVG-directly-emitted' in render_types:
            existing_file = getattr(self, 'drawing-directly-emitted.html', None)
            if existing_file:
                 self.manage_delObjects('drawing-directly-emitted.html')

            self.invokeFactory('Document', id='drawing-directly-emitted.html')
            html_file = self['drawing-directly-emitted.html']
            html_file.edit(text_format='html',text=''.join(teaching_schedule.getHTMLForContainingSVG('drawing-directly-emitted')))
            #def edit(self, text_format, text, file='', safety_belt=''):

            existing_file = getattr(self, 'drawing-directly-emitted.svg', None)
            if existing_file:
                 self.manage_delObjects('drawing-directly-emitted.svg')

            self.invokeFactory('File', id='drawing-directly-emitted.svg')
            svg_file = self['drawing-directly-emitted.svg']
            svg_file.update_data(data=''.join(teaching_schedule.getSVG()))
        
        #     self.invokeFactory('File', id='svgecmascript.js')
        #     ecmascript_file = self['svgecmascript.js']
        #     ecmascript_string = string.join(drawing.getEcmascript())
        #ecmascript_file.file_edit(file=dummy.File('svgecmascript.js', data=string.join(drawing.getEcmascript())))
        #     ecmascript_file.update_data(data=ecmascript_string)

        if 'PDF' in render_types:
            existing_file = getattr(self, 'drawing.pdf', None)
            if existing_file:
                 self.manage_delObjects('drawing.pdf')

            canvas = Canvas('drawing.pdf', pagesize=(drawing.width, drawing.height))
            canvas.setFont('Times-Roman', 12)
            drawing.drawOn(canvas, 0, 0)
            pdf_content = str(canvas.getpdfdata())
            self.invokeFactory('File', id='drawing.pdf')
            pdf_file = self['drawing.pdf']
            pdf_file.update_data(data=pdf_content)

        if 'PS' in render_types:
            existing_file = getattr(self, 'drawing.ps', None)
            if existing_file:
                 self.manage_delObjects('drawing.ps')

            from reportlab.graphics import renderPS
            render_result = renderPS.drawToString(drawing)
            self.invokeFactory('File', id='drawing.ps')
            ps_file = self['drawing.ps']
            ps_file.update_data(data=render_result)

        if 'HTML' in render_types:
            existing_file = getattr(self, 'drawing.html', None)
            if existing_file:
                 self.manage_delObjects('drawing.html')

            self.invokeFactory('Document', id='drawing.html')
            html_file = self['drawing.html']
            html_file.edit(text_format='html',text=''.join(teaching_schedule.getHTML()))

registerType(TeachingSchedule, PROJECTNAME)

