# Version 0.1
#
#  TeachingSchedule is copyright 2005 Jeffrey G. Pittman.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#  MA 02111-1307 USA

import csv, string, cStringIO

import os, sys, math, random

from reportlab.lib import colors
from reportlab.pdfgen.canvas import Canvas

from reportlab.graphics.shapes import Group
from reportlab.graphics.shapes import Drawing

import csv
import calendar
# the dateutil functionality should perhaps be added:
#from dateutil.relativedate import *
#from dateutil.rrule import *
#from dateutil.parser import *
from datetime import date
from datetime import datetime
from datetime import timedelta

from Products.TeachingSchedule.utils import getrlcolor
from Products.TeachingSchedule.utils import colorstr
from Products.TeachingSchedule.utils import wordWrap
from Products.TeachingSchedule.utils import rgb256

from Products.TeachingSchedule.graphics import Line
from Products.TeachingSchedule.graphics import DashedLine
from Products.TeachingSchedule.graphics import Circle
from Products.TeachingSchedule.graphics import Rectangle
from Products.TeachingSchedule.graphics import IDRectangle
from Products.TeachingSchedule.graphics import Text
from Products.TeachingSchedule.graphics import TransformedText
from Products.TeachingSchedule.graphics import AlignedText
from Products.TeachingSchedule.graphics import TransformedAndAlignedText

from reportlab.graphics.shapes import Line as RLLine
from reportlab.graphics.shapes import Rect as RLRect
from reportlab.graphics.shapes import Circle as RLCircle
from reportlab.graphics.shapes import Ellipse as RLEllipse
from reportlab.graphics.shapes import PolyLine as RLPolyLine
from reportlab.graphics.charts.textlabels import Label

class TimeRange:
    def __init__(self, start, end):
        self.start = start
        self.end = end

class RecurrenceInterval:
    def __init__(self, interval):
        self.interval = interval

    def isHourly(self):
        if self.interval == 'HOURLY':
            return 1
        else:
            return 0

    def isDaily(self):
        if self.interval == 'DAILY':
            return 1
        else:
            return 0

    def isWeekly(self):
        if self.interval == 'WEEKLY':
            return 1
        else:
            return 0

    def isMonthly(self):
        if self.interval == 'MONTHLY':
            return 1
        else:
            return 0

    def isYearly(self):
        if self.interval == 'YEARLY':
            return 1
        else:
            return 0

class Event:
    def __init__(self, year, month, day, hour, minute, 
                 duration, description, color):
        self.year = year
        self.month = month
        self.day = day
        self.hour = hour
        self.minute = minute
        self.duration = duration
        self.description = description
        self.color = color

        self.start = date(year, month, day)
        self.end = self.start + self.duration

class RecurringEvent:
    def __init__(self, year, month, day, hour, minute, 
                       duration, recurrence_interval, start, end,
                       description, color):
        self.year = year
        self.month = month
        self.day = day
        self.hour = hour
        self.minute = minute
        self.duration = duration
        self.recurrence_interval = RecurrenceInterval(recurrence_interval)
        self.start = start
        self.end = end
        self.description = description
        self.color = color
        self.events = self.generateEvents()

    def generateEvents(self):
        events = []
        if self.recurrence_interval.isWeekly():
            day_of_week = calendar.weekday(self.year,self.month,self.day)
            for target_weekday in self.getInstancesOfWeekday(day_of_week):
                events.append(Event(target_weekday.year, 
                                    target_weekday.month, 
                                    target_weekday.day, 
                                    self.hour, 
                                    self.minute, 
                                    self.duration,
                                    self.description,
                                    self.color))
        else:
            start_month = -1
            for year in range(self.start.year, self.end.year+1):
                if self.recurrence_interval.isYearly():
                    events.append(Event(year, self.month, self.day, self.hour, 
                                        self.minute, self.duration,
                                        self.description, self.color))
                else:
                    if start_month == -1:
                        start_month = self.start.month
                        if year == self.end.year:
                            end_month = self.end.month
                        else:
                            end_month = 12
                    elif year == self.end.year:
                        if year == self.start.year:
                            start_month = self.start.month
                        else:
                            start_month = 1
                        end_month = self.end.month
                    else:
                        start_month = 1
                        end_month = 12
                    start_day = -1
                    for month in range(start_month, end_month+1):
                        if self.recurrence_interval.isMonthly():
                            events.append(Event(year, month, self.day, self.hour, 
                                                self.minute, self.duration,
                                                self.description, self.color))
                        else:
                            if start_day == -1:
                                start_day = self.start.day
                                if month == self.end.month:
                                    end_day = self.end.day
                                else:
                                    end_day = calendar.monthrange(year, month)[1]
                            elif month == self.end.month:
                                if month == self.start.month:
                                    start_day = self.start.day
                                else:
                                    start_day = 1
                                end_day = self.end.day
                            else:
                                start_day = 1
                                end_day = calendar.monthrange(year, month)[1]
                            start_hour = -1
                            for day in range(start_day, end_day+1):
                                if self.recurrence_interval.isDaily():
                                    events.append(Event(year, month, day, self.hour, 
                                                        self.minute,
                                                        self.duration,
                                                        self.description,
                                                        self.color))
                                elif self.recurrence_interval.isHourly():
                                    if start_hour == -1:
                                        start_hour = self.start.hour
                                        if day == self.end.day:
                                            end_hour == self.end.hour
                                        else:
                                            end_hour = 24
                                    elif day == self.end.day:
                                        if day == self.start.day:
                                            start_hour = self.start.hour
                                        else:
                                            start_hour = 1
                                        end_hour = self.end.hour
                                    else:
                                        start_hour = 1
                                        end_hour = 24
                                    start_minute = -1
                                    for hour in range(start_hour, end_hour+1):
                                        events.append(Event(year, month, day, hour, 
                                                            self.minute,
                                                            self.duration,
                                                            self.description,
                                                            self.color))
                                else:
                                    pass
        return events

    def getEvents(self):
        return self.events
                                    
    def getInstancesOfWeekday(self, weekday):
        weekdays = []
        one_day = date(2005,1,1) - date(2004,12,31)
        first_day = date(self.year, self.month, self.day)
        weekdays.append(first_day)
        day = first_day + one_day
        while day != self.end:
            if calendar.weekday(day.year, day.month, day.day) == weekday:
                weekdays.append(day)
            day += one_day
            
        #days_in_current_month = calendar.monthrange(year, month, day)
        #for year in range(self.start.year, self.end.year+1):
        #    if year == self.start.year:
        #        for month in range(self.start.month, 13):
        #            if month == self.start.month:
        #                for day in range(self.start.day, calendar.monthrange(year, month)[1]+1):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #            elif month == self.end.month:
        #                for day in range(1, self.end.day):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #            else:
        #                for day in range(1, calendar.monthrange(year, month)[1]+1):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #    elif year == self.end.year:
        #        for month in range(1, self.end.month+1):
        #            if month == self.end.month:
        #                for day in range(1, self.day+1):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #            else:
        #                for day in range(1, calendar.monthrange(year, month)[1]+1):
        #                    if calendar.weekday(year, month, day) == weekday:
        #                        weekdays.append(date(year, month, day))
        #    else:
        #        for month in range(1, 13):
        #            for day in range(1, calendar.monthrange(year, month)[1]+1):
        #                if calendar.weekday(year, month, day) == weekday:
        #                    weekdays.append(date(year, month, day))
        return weekdays

class DayPortion:
    def __init__(self, portion):
        self.portion = portion

    def isMorning(self):
        if self.portion == 'MORNING':
            return 1
        else:
            return 0

    def isAfternoon(self):
        if self.portion == 'AFTERNOON':
            return 1
        else:
            return 0

    def isDay(self):
        if self.portion == 'DAY':
            return 1
        else:
            return 0

class VacationDay:
    def __init__(self, year, month, day, day_portion_off, reason, username):
        self.date = date(year, month, day)
        self.day_portion = DayPortion(day_portion_off)
        self.reason = reason
        self.username = username

    def getStart(self):
        if self.day_portion.isDay():
            return 7
        if self.day_portion.isMorning():
            return 7
        if self.day_portion.isAfternoon():
            return 12
        return 7

    def getDuration(self):
        if self.day_portion.isDay():
            return timedelta(hours=10)
        if self.day_portion.isMorning():
            return timedelta(hours=5)
        if self.day_portion.isAfternoon():
            return timedelta(hours=5)
        return timedelta(hours=10)

class Person:
    def __init__(self, username, realname, groupname, vacation_days, color):
        self.username = username
        self.realname = realname
        self.groupname = groupname
        self.vacation_days = []
        self.addVacationDays(vacation_days)
        self.color = color

    def addVacationDay(self, year, month, day, day_portion_off, reason):
        self.vacation_days.append(VacationDay(year, month, day, day_portion_off, reason, self.username))

    def addVacationDayObject(self, vacation_day):
        self.vacation_days.append(vacation_day)

    def addVacationDays(self, vacation_days):
        for vacation_day in vacation_days:
            self.addVacationDayObject(vacation_day)

    def getVacationForDate(self, search_date):
        vacation_for_date = []
        for vacation_day in self.vacation_days:
            if vacation_day.date == search_date:
                return vacation_day
        return None

class FacultyMember(Person):
    def __init__(self, username, realname, groupname, 
                 vacation_days, classes, color):
        Person.__init__(self, username, realname, groupname, vacation_days, color)
        self.classes = classes

    def getClassTimes(self):
        return self.classes

    def getClassTimeEvents(self):
        events = []
        for class_time in self.classes:
            for event in class_time.getEvents():
                events.append(event)
        return events

class EventContainer:
    def __init__(self):
        self.events = []
        self.recurring_events = []
        self.person_associations = {}
        self.group_associations = {}

    def addEvent(self, event):
        self.events.append(event)

    def addRecurringEvent(self, recurring_event):
        self.recurring_events.append(recurring_event)

    def getAllEvents(self):
        all_events = []
        for event in self.events:
            all_events.append(event)
        for recurring_event in self.recurring_events:
            for event in recurring_event.getEvents():
                all_events.append(event)
        return all_events

    def getAllEventsForDate(self, search_date):
        all_events = []
        for event in self.getAllEvents():
            if event.start == search_date:
                all_events.append(event)
        return all_events

    def getAllEventsForDateForPerson(self, search_date, username):
        all_events = []
        for event in self.person_associations[username]:
            if event.start == search_date:
                all_events.append(event)
        return all_events

    def getAllEventsForDateForGroup(self, search_date, group):
        all_events = []
        for event in self.person_associations[group]:
            if event.start == search_date:
                all_events.append(event)
        return all_events

    def addAssociationsForPersonToEvents(self, username, events):
        for event in events:
            if self.person_associations.has_key(username):
                self.person_associations[username].append(event)
            else:
                self.person_associations[username] = []
                self.person_associations[username].append(event)
        
    def addAssociationsForGroupToEvents(self, username, events):
        for event in events:
            if self.group_associations.has_key(username):
                self.group_associations[username].append(event)
            else:
                self.group_associations[username] = []
                self.group_associations[username].append(event)
        
    def addAssociationForPersonToRecurringEvent(self, username, recurring_event):
        self.addAssociationsForPersonToEvents(username, recurring_event.getEvents())
        
    def addAssociationForGroupToRecurringEvent(self, groupname, recurring_event):
        self.addAssociationsForPersonToEvents(groupname, recurring_event.getEvents())
        
    def getMonthNumber(self, month_name):
        month_names = { 'January':1,
                        'February':2,
                        'March':3,
                        'April':4,
                        'May':5,
                        'June':6,
                        'July':7,
                        'August':8,
                        'September':9,
                        'October':10,
                        'November':11,
                        'December':12 }

        return month_names[month_name]

    def getMonthName(self, month):
        month_names = { 1:'January',
                        2:'February',
                        3:'March',
                        4:'April',
                        5:'May',
                        6:'June',
                        7:'July',
                        8:'August',
                        9:'September',
                        10:'October',
                        11:'November',
                        12:'December' }
    
        return month_names[month]
    
    def getWeekdayAbbreviation(self, year, month, day):
        weekday_abbreviations = { 0:'MON',
                                  1:'TUE',
                                  2:'WED',
                                  3:'THU',
                                  4:'FRI',
                                  5:'SAT',
                                  6:'SUN' }

        return weekday_abbreviations[calendar.weekday(year, month, day)]

class PersonContainer:
    def __init__(self):
        self.persons = []

    def addPerson(self, person):
        self.persons.append(person)

    def getVacationsForPersons(persons, search_start, search_end):
        days_and_persons = {}
        mornings_and_persons = {}
        afternoons_and_persons = {}
        day_delta = timedelta(days=1)
        day = search_start
        while day <= search_end:
            days_and_persons[day] = []
            mornings_and_persons[day] = []
            afternoons_and_persons[day] = []
            for person in persons:
                vacation_day = person.getVacationForDate(day)
                if vacation_day is not None:
                    if vacation_day.day_portion.isDay():
                        days_and_persons[day].append(person)
                    elif vacation_day.day_portion.isMorning():
                        mornings_and_persons[day].append(person)
                    else:
                        afternoons_and_persons[day].append(person)
            day += day_delta
        return (days_and_persons, mornings_and_persons, afternoons_and_persons)

    def getAvailableDaysForPersons(day_portion, persons, search_start, search_end):
        vacation_days_and_persons, vacation_mornings_and_persons, vacation_afternoons_and_persons = self.getVacationsForPersons(persons, search_start, search_end)
        if day_portion.isDay(): 
            available_days = []
            day = search_start
            while day <= search_end:
                if len(vacation_days_and_persons[day]) == 0:
                    available_days.append(day)
            return available_days
        elif day_portion.isMorning(): 
            available_mornings = []
            day = search_start
            while day <= search_end:
                if len(vacation_mornings_and_persons[day]) == 0:
                    available_mornings.append(day)
                day += day_delta
            return available_mornings
        elif day_portion.isAfternoon(): 
            available_afternoons = []
            day = search_start
            while day <= search_end:
                if len(vacation_afternoons_and_persons[day]) == 0:
                    available_afternoons.append(day)
                day += day_delta
            return available_afternoons
        return []

    def getFirstAvailableDayForPersons(day_portion, persons, search_start, search_end):
        available_days = self.getAvailableDaysForPersons(day_portion, persons, search_start, search_end)
        if len(available_days) == 0:
            return None
        else:
            return available_days[0]

    def getBestAvailableForPersons(day_portion, persons, search_start, search_end):
        vacation_days_and_persons, vacation_mornings_and_persons, vacation_afternoons_and_persons = self.getVacationsForPersons(persons, search_start, search_end)
        search_dict = {}
        if day_portion.isDay(): 
            search_dict = vacation_days_and_persons
        elif day_portion.isMorning(): 
            search_dict = vacation_mornings_and_persons
        elif day_portion.isAfternoon(): 
            search_dict = vacation_afternoons_and_persons
        available_days = []
        day = search_start
        max_day = day
        while day <= search_end:
            if len(search_dict[day]) > len(search_dict[max_day]):
                max_day = day
        return (max_day, search_dict[max_day])

class PersonAndEventDrawing:
    def __init__(self, left_margin, right_margin, bottom_margin, top_margin, 
                 first_day, last_day, html_filename, pdf_filename):
        self.event_container = EventContainer()
        self.person_container = PersonContainer()
        self.total_width = 0
        self.total_height = 0
        self.left_margin = left_margin
        self.right_margin = right_margin
        self.bottom_margin = bottom_margin
        self.top_margin = top_margin
        self.first_day = first_day
        self.last_day = last_day
        self.html_filename = html_filename
        self.pdf_filename = pdf_filename

    def addPerson(self, person):
        self.person_container.addPerson(person)

    def addEvent(self, event):
        self.event_container.addEvent(event)

    def addRecurringEvent(self, recurring_event):
        self.event_container.addRecurringEvent(recurring_event)

    def getDrawing(self):
        pass

    def write_html(self):
        html_lines = self.getHTML() 
        if html_lines:
            htmlname = self.html_filename
            file = open(htmlname,'w')
            file.writelines(html_lines) 
            file.close()

    def getHTML(self):
        pass

    def writePDF(self):
        #pdfPath = os.path.splitext('panelcalendar.pdf')
        drawing = self.getDrawing()
        if drawing:
            c = Canvas(self.pdf_filename, pagesize=(self.total_width, self.total_height))
            c.setFont('Times-Roman', 12)
            drawing.drawOn(c, 0, 0)
            c.save()

    def getPDFData(self):
        drawing = self.getDrawing()
        if drawing:
            c = Canvas(self.pdf_filename, pagesize=(self.total_width, self.total_height))
            c.setFont('Times-Roman', 12)
            drawing.drawOn(c, 0, 0)
            return c.getpdfdata()

class DayPlanningCalendar(PersonAndEventDrawing):
    """
    The basic structure of SVG emitting methods is a modification of:
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/325823,
    whereby the strarray() method emits a list of lines that are
    added to a growing SVG file.

    SVG help at www.carto.net was also very useful.    
    """
    def __init__(self, left_margin, right_margin, bottom_margin, top_margin, 
                 first_day, last_day, html_filename, pdf_filename,
                 day_width, hour_height, start_hour, end_hour):
        PersonAndEventDrawing.__init__(self, left_margin, right_margin, bottom_margin, top_margin, 
                                             first_day, last_day, html_filename, pdf_filename)
        self.day_width = day_width
        self.hour_height = hour_height
        self.start_hour = start_hour
        self.end_hour = end_hour

        # hard-coded for now
        self.day_spacing = 2

    def yUp(self, y):
        return self.total_height - float(y)

    def getHTMLForContainingSVG(self, name):        
        var = ["<body>\n",
               "<ul>\n",
               "    <li><b>Zooming</b>: use the CTRL-Key (or Apple-Key) to drag a rectangle and zoom in.\n",
               "        <br />Use the right mouse button (CTRL-Key plus mouse click on the Mac) to get a context menu, <br />where you can choose step-by-step zooming and can go back to the original extent.</li>\n",
               "    <li><b>Zoom Out</b>: like above, but with SHIFT-Key pressed.</li>\n",
               "    <li><b>Panning</b>: Press the ALT-Key and move the mouse cursor while a hand-icon appears.</li>\n",
               "    <li><b>Anti-Aliasing</b>: May be turned on and off with the help of the context menu. (Higher Quality).</li>\n",
               "</ul>\n",
               "<h2>SVG Teaching Schedule</h2>\n",
               "          <embed width=\"100%%\" height=\"660px\" src=\"%s.svg\" type=\"image/svg+xml\" />\n" % name,
               "</body>\n"]
               #"              <b>For viewing SVG here, perhaps a browser update would help.<br />Perhaps not, as SVG browser support is now incomplete.</b>",
               #"          <object width=\"600\" height=\"660\" data=\"%s.svg\" type=\"image/svg+xml\" >\n" % name,
               #"          <iframe width=\"600\" height=\"660\" src=\"%s.svg\" frameborder=\"0\" marginwidth=\"0\" marginheight=\"0\" >\n" % name,
               #"              <embed width=\"600\" height=\"660\" src=\"%s.svg\" type=\"image/svg+xml\" />\n" % name,
            #["  <script type=\"text/javascript\">\n",
            #["<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n",
             #"<html>\n",
             #"<head>\n",
             #"  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n",
             #"  <meta http-equiv=\"imagetoolbar\" content=\"no\">\n",
             #"  <meta name=\"keywords\" content=\"geologic time scale svg scalable vector graphics\">\n",
             #"  <meta name=\"description\" content=\"SVG Teaching Schedule\">\n",
             #"  <style type=\"text/css\">\n",
             #"      body {background-color:#ffef95;font-family:Arial,Helvetica,Geneva,sans-serif;color:#B42D25;}\n",
             #"      .bigger {font-weight:bold;}\n",
             #"      .smaller {font-size:small;}\n",
             #"  </style>\n",
             #"      function hilite_elem(checkbox, element_name){\n",
             #"        // For each element, get the element's style object, then set\n",
             #"        // its visibility according to the state of the checkbox.\n",
             #"        var svgobj = document.embeds['teachingschedule'].getSVGDocument().getElementById(element_name);\n",
             #"        if (!checkbox.checked){\n",
             #"            // Hide layer.\n",
             #"            svgobj.setAttributeNS(null,'visibility','hidden');\n",
             #"        } else {\n",
             #"            // Show layer.\n",
             #"            svgobj.setAttributeNS(null,'visibility','visible');\n",
             #"        }\n",
             #"      }\n",
             #"  </script>\n",
             #"  <title>SVG Teaching Schedule</title>\n",
             #"</head>\n",
             #"          <object width=\"600\" height=\"660\" data=\"%s.svg\" name=\"teachingschedule\" type=\"image/svg+xml\"/>\n" % name,
             #"            <br>\n",
             #"            <form name=\"hilite_form\" action=\"\">\n",
             #"              <table border=\"0\" cellpadding=\"0\" cellspacing=\"2\">\n",
             #"                <tr>\n",
             #"                  <td colspan=\"2\">Visible Layers\n",
             #"                  </td>\n",
             #"                </tr>\n",
             #"                <tr>\n",
             #"                  <td><input type=\"checkbox\" value=\"\" onclick=\"hilite_elem(this,'bar_scale')\">&nbsp;Bar Scale</TD>\n",
             #"                  <td><input type=\"checkbox\" value=\"\" onclick=\"hilite_elem(this,'header_labels')\">&nbsp;Header Labels</TD>\n",
             #"                </tr>\n",
             #"               </table>\n",
             #"               <script type=\"text/javascript\">\n",
             #"                 // Make sure all checkboxes are checked whenever the page is reloaded in the browser.\n",
             #"                 for (var i = 0;  i < document.hilite_form.elements.length;  i++)\n",
             #"                   if (document.hilite_form.elements[i].type == 'checkbox'){\n",
             #"                     document.hilite_form.elements[i].checked = true;\n",
             #"                   }  \n",
             #"               </script>\n",
             #"            </form>\n",
             #"      <table border=\"0\" cellpadding=\"8\" cellspacing=\"0\">\n",
             #"        <tr>\n",
             #"          <td align=\"center\" valign=\"top\">\n",
             #"          </td>\n",
             #"          <td valign=\"top\">\n",
             #"            <p>Use mouse functions to change display:</p>\n",
             #"            <ul>\n",
             #"              <li><span class=\"bigger\">Layers (if enabled)</span>: Use the checkboxes at bottom to toggle items on and off.</li>\n",
             #"            </ul>\n",
             #"            <p>Use keyboard functions to change display:</p>\n",
             #"            <ul>\n",
             #"              <li><span class=\"bigger\">Zooming</span>: use the CTRL-Key (or Apple-Key) to drag a rectangle and zoom in.\n",
             #"                  <br>Use the right mouse button (CTRL-Key&nbsp;+ mouse click on the Mac) to get a context menu, where you can choose step-by-step zooming and can go back to the original extent.</li>\n",
             #"              <li><span class=\"bigger\">Zoom Out</span>: like above, but with SHIFT-Key pressed.</li>\n",
             #"              <li><span class=\"bigger\">Panning</span>: Press the ALT-Key and move the mouse cursor while a \"hand-icon\" appears.</li>\n",
             #"              <li><span class=\"bigger\">Anti-Aliasing</span>: May be turned on and off with the help of the context menu. (&quot;Higher Quality&quot;).</li>\n",
             #"            </ul>\n",
             #"          </td>\n",
             #"        </tr>\n",
             #"      </table>\n",
             #"</body></html>\n"]
        return var

    def getHTML(self):
        var = ["<body>\n"]

        day_delta = timedelta(days = 1)
        item_width = 0
        day = self.first_day
        while day <= self.last_day:
            var += "<H1>On %d/%d</H1>\n" % (day.month, day.day)
            var_lines = len(var)
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                if len(events_for_day) > 0:
                    var += "<dl>\n"
                    var += "    <dt>%s teaches:</dt>\n" % person.username
                    for event in events_for_day:
                        var += "    <dd>%s at %d:%2d</dd>\n" % (event.description, event.hour, event.minute)
                    var += "</dl>\n"

            if var_lines == len(var):
                var += "<p>No classes taught</p>\n"

            day += day_delta

        var += "</body>\n"

        return var

    def getCSS(self):
        css = \
          ["\n",
           "        <style type=\"text/css\">\n",
           "            <![CDATA[\n",
           "                body {font-family:Arial,Helvetica,Geneva,sans-serif;}\n",
           "                .str0 {\n",
           "                    stroke:#000000;\n",
           "                    stroke-width:0.1\n",
           "                }\n",
           "            ]]>\n",
           "        </style>\n",
           "\n"]
        return css

    def getEcmascript(self):
        """
        Note: at one time there were three description lines in the mouseover.
        These are commented out. This could be useful to have more info
            in the tooltip.
        """
        ecmascript = \
            ["        <script language=\"Javascript\"><![CDATA[\n",
             "        // This script from http://svg-whiz.com/svg/Tooltip.svg\n",
             "        var SVGDocument = null;\n",
             "        var SVGRoot = null;\n",
             "        var SVGViewBox = null;\n",
             "        var svgns = 'http://www.w3.org/2000/svg';\n",
             "        var xlinkns = 'http://www.w3.org/1999/xlink';\n",
             "        var toolTip = null;\n",
             "        var TrueCoords = null;\n",
             "        var tipBox = null;\n",
             "        var tipText = null;\n",
             "        var tipTitle = null;\n",
             #"        var tipDesc1 = null;\n",
             #"        var tipDesc2 = null;\n",
             #"        var tipDesc3 = null;\n",
             "\n",
             "        var lastElement = null;\n",
             "        var titleText = '';\n",
             #"        var titleDesc1 = '';\n",
             #"        var titleDesc2 = '';\n",
             #"        var titleDesc3 = '';\n",
             "        function Init(evt)\n",
             "        {\n",
             "        SVGDocument = evt.target.ownerDocument;\n",
             "        SVGRoot = document.documentElement;\n",
             "        TrueCoords = SVGRoot.createSVGPoint();\n",
             "        \n",
             "        toolTip = SVGDocument.getElementById('ToolTip');\n",
             "        tipBox = SVGDocument.getElementById('TipBox');\n",
             "        tipText = SVGDocument.getElementById('TipText');\n",
             "        tipTitle = SVGDocument.getElementById('TipTitle');\n",
             #"        tipDesc1 = SVGDocument.getElementById('tipDesc1');\n",
             #"        tipDesc2 = SVGDocument.getElementById('tipDesc2');\n",
             #"        tipDesc3 = SVGDocument.getElementById('tipDesc3');\n",
             "        };\n",
             "\n",
             "        function GetTrueCoords(evt)\n",
             "        {\n",
             "           // find the current zoom level and pan setting, and adjust the reported\n",
             "           //    mouse position accordingly\n",
             "           var newScale = SVGRoot.currentScale;\n",
             "           var translation = SVGRoot.currentTranslate;\n",
             "           TrueCoords.x = (evt.clientX - translation.x)/newScale;\n",
             "           TrueCoords.y = (evt.clientY - translation.y)/newScale;\n",
             "        };\n",
             "\n",
             "        function ShowTooltip(evt, turnOn)\n",
             "        {\n",
             "           try\n",
             "           {\n",
             "              if (!evt || !turnOn)\n",
             "              {\n",
             "                 toolTip.setAttributeNS(null, 'display', 'none');\n",
             "              }\n",
             "              else\n",
             "              {\n",
             "                 var tipScale = 1/SVGRoot.currentScale;\n",
             "                 var textWidth = 0;\n",
             "                 var tspanWidth = 0;\n",
             "                 var boxHeight = 20;\n",
             "\n",
             "                 tipBox.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );\n",
             "                 tipText.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );\n",
             "\n",
             "                 var targetElement = evt.target;\n",
             "\n",
             "                 if ( lastElement != targetElement )\n",
             "                 {\n",
             "                    var targetTitle = targetElement.getElementsByTagName('title').item(0);\n",
             "                    if ( targetTitle )\n",
             "                    {\n",
             "                       titleText = targetTitle.firstChild.nodeValue;\n",
             "                       tipTitle.firstChild.nodeValue = titleText;\n",
             "                    }\n",
             "\n",
             #"                    var targetDesc1 = targetElement.getElementsByTagName('desc1').item(0);\n",
             #"                    if ( targetDesc1 )\n",
             #"                    {\n",
             #"                       titleDesc1 = targetDesc1.firstChild.nodeValue;\n",
             #"                       tipDesc1.firstChild.nodeValue = titleDesc1;\n",
             #"                    }\n",
             #"\n",
             #"                    var targetDesc2 = targetElement.getElementsByTagName('desc2').item(0);\n",
             #"                    if ( targetDesc2 )\n",
             #"                    {\n",
             #"                       titleDesc2 = targetDesc2.firstChild.nodeValue;\n",
             #"                       tipDesc2.firstChild.nodeValue = titleDesc2;\n",
             #"                    }\n",
             #"\n",
             #"                    var targetDesc3 = targetElement.getElementsByTagName('desc3').item(0);\n",
             #"                    if ( targetDesc3 )\n",
             #"                    {\n",
             #"                       titleDesc3 = targetDesc3.firstChild.nodeValue;\n",
             #"                       tipDesc3.firstChild.nodeValue = titleDesc3;\n",
             #"                    }\n",
             "                 }\n",
             "\n",
             "                 var xPos = TrueCoords.x + (10 * tipScale);\n",
             "                 var yPos = TrueCoords.y + (10 * tipScale);\n",
             "\n",
             "                 //return rectangle around object as SVGRect object\n",
             "                 var outline = tipText.getBBox();\n",
             "                 tipBox.setAttributeNS(null, 'width', Number(outline.width) + 10);\n",
             "                 tipBox.setAttributeNS(null, 'height', Number(outline.height) + 10);\n",
             "\n",
             "                 toolTip.setAttributeNS(null, 'transform', 'translate(' + xPos + ',' + yPos + ')');\n",
             "                 toolTip.setAttributeNS(null, 'display', 'inline');\n",
             "              }\n",
             "           }\n",
             "           catch(er){}\n",
             "         };\n",
             "        ]]></script>\n"]
        return ecmascript

    def getOpenerSection(self):
        opener_section = \
          ["<?xml version=\"1.0\"?>\n",
           "<svg  id=\"teachingschedule\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" height=\"%d\" width=\"%d\">\n" % (self.total_height,self.total_width)]
           #"<svg  id=\"teachingschedule\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" onload=\"Init(evt)\" onmousemove=\"GetTrueCoords(evt); ShowTooltip(evt, true)\" onmouseout=\"ShowTooltip(evt, false)\" height=\"%d\" width=\"%d\">\n" % (self.total_height,self.total_width)]
        return opener_section
                      
    def getDefsSection(self):
        defs_section = ["    <defs>\n"]
        defs_section += self.getCSS()
        defs_section += self.getEcmascript()
        defs_section += ["    </defs>\n"]
        return defs_section
                    
    def getBottomPopupSection(self):
        bottom_svg = \
          ["    <g id='ToolTip' opacity='0.8' display='none' pointer-events='none'>\n",
           "        <rect id='TipBox' x='0' y='5' width='88' height='20' rx='2' ry='2' fill='white' stroke='black'/>\n",
           "        <text id='TipText' x='5' y='20' font-family='Arial' font-size='12'>\n",
           "            <tspan id='TipTitle' x='5' font-weight='bold'> </tspan>\n",
           #"         <tspan id='tipDesc1' x='5' dy='1.2em' fill='blue'> </tspan>\n",
           #"         <tspan id='tipDesc2' x='5' dy='1.2em' fill='blue'> </tspan>\n",
           #"         <tspan id='tipDesc3' x='5' dy='1.2em' fill='blue'> </tspan>\n",
           "        </text>\n",
           "    </g>\n"]
        return bottom_svg
    
    def getCloserSection(self):
        closer_section = ["</svg>\n"]
        return closer_section
        
    def getTeachingScheduleSection(self):
        # For SVG, the 0,0 point is in the upper left corner, as compared
        # to the lower left corner for the reportlab toolkit. We handle
        # this difference in the y-axis by use of the yUp() method.
      
        # make a preliminary pass to determine item_width
        day_delta = timedelta(days = 1)
        item_width = 0
        day = self.first_day
        while day <= self.last_day:
            person_count_with_events = 0
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                if len(events_for_day) > 0:
                    person_count_with_events += 1
                    item_width += self.day_width + self.day_spacing
            day += day_delta
            if not person_count_with_events:
                item_width += self.day_width + self.day_spacing

        panel_width = item_width
        self.total_width = self.left_margin + panel_width + self.right_margin

        #panel_height = (self.end_hour - self.start_hour) * self.hour_height
        panel_height = 24 * self.hour_height
        self.total_height = self.bottom_margin + panel_height + self.top_margin
    
        var = []

        # paint background rects for days with events
        day_delta = timedelta(days = 1)
        item_width = 0
        day = self.first_day
        while day <= self.last_day:
            person_count_with_events = 0
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                if len(events_for_day) > 0:
                    person_count_with_events += 1
                    item_width += self.day_width + self.day_spacing
            day += day_delta
            if not person_count_with_events:
                item_width += self.day_width + self.day_spacing
            else:
                x_bg_rect = item_width - (person_count_with_events * (self.day_width + self.day_spacing)) - (person_count_with_events/2)
                y_bg_rect = self.start_hour * self.hour_height
                bg_rect_width = person_count_with_events * (self.day_width + self.day_spacing)
                bg_rect_height = (self.end_hour - self.start_hour) * self.hour_height
                rectangle = Rectangle((self.left_margin + x_bg_rect, 
                                       self.yUp(self.bottom_margin + y_bg_rect + bg_rect_height)),
                                      bg_rect_width,
                                      bg_rect_height,
                                      0.1,
                                      rgb256(colors.lightgrey.rgb()),
                                      rgb256(colors.black.rgb()))
                var += rectangle.strarray()

        # draw lines and labels for hours
        panel_width = item_width
        for hour in range(self.start_hour, self.end_hour+1):
            if hour == 12:
                strokeWidth = 1.0
            else:
                strokeWidth = 0.1
            line = Line((self.left_margin - 2, 
                         self.yUp(self.bottom_margin + (hour * self.hour_height))),
                        (self.left_margin + panel_width + 2, 
                         self.yUp(self.bottom_margin + (hour * self.hour_height))),
                        strokeWidth,
                        rgb256(colors.black.rgb()))
            var += line.strarray()

            label = AlignedText((self.left_margin - 4, 
                                 self.yUp(self.bottom_margin + (hour * self.hour_height))),
                                "%d:00" % hour,
                                rgb256(colors.black.rgb()),
                                14,
                                "end")
            var += label.strarray()

            label = AlignedText((self.left_margin + panel_width + 4, 
                                 self.yUp(self.bottom_margin + (hour * self.hour_height))),
                                "%d:00" % hour,
                                rgb256(colors.black.rgb()),
                                14,
                                "start")
            var += label.strarray()

        # draw events
        day_delta = timedelta(days = 1)
        x = 0
        day = self.first_day
        while day <= self.last_day:
            person_count_with_events = 0
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                #if len(events_for_day) > 0:
                    #line = DashedLine((self.left_margin + (x + ((self.day_width + self.day_spacing) / 2) - 1), 
                                       #self.yUp(self.bottom_margin + (self.start_hour * self.hour_height - 2))),
                                      #(self.left_margin + (x + ((self.day_width + self.day_spacing) / 2) - 1), 
                                       #self.yUp(self.bottom_margin + (self.end_hour * self.hour_height))),
                                      #1,
                                      #rgb256(colors.black.rgb()),
                                      #'1,2')
                    #var += line.strarray()

                for event in events_for_day:
                    duration_hours = float(event.duration.seconds) / 3600.0
                    y = int((float(event.hour) + (float(event.minute) / 60.0)) * float(self.hour_height))
                    height = duration_hours * float(self.hour_height)
                    rectangle = Rectangle((self.left_margin + x, 
                                           self.yUp(self.bottom_margin + y + height)),
                                          self.day_width,
                                          height,
                                          0.5,
                                          rgb256(event.color.rgb()),
                                          rgb256(colors.black.rgb()))
                    var += rectangle.strarray()

                if len(events_for_day) > 0:
                    person_count_with_events += 1
    
                    label = TransformedAndAlignedText((self.left_margin + (x + (self.day_width/2) - 1), 
                                                       self.yUp(self.bottom_margin + ((self.start_hour * self.hour_height) - 4))),
                                                      "%s" % person.username,
                                                      rgb256(colors.black.rgb()),
                                                      14,
                                                      -90,
                                                      'end')
                    var += label.strarray()

                    x += self.day_width + self.day_spacing
    
            if not person_count_with_events:
                label = TransformedAndAlignedText((self.left_margin + (x + (self.day_width/2) + 1), 
                                                   self.yUp(self.bottom_margin + ((self.end_hour * self.hour_height) + 4))),
                                                  "%s: %d/%d" % (self.event_container.getWeekdayAbbreviation(day.year, 
                                                                                                           day.month, 
                                                                                                           day.day),
                                                                 day.month,
                                                                 day.day),
                                                  rgb256(colors.black.rgb()),
                                                  14,
                                                  -90,
                                                  'start')
                var += label.strarray()

                x += self.day_width + self.day_spacing
            else:
                width = person_count_with_events * (self.day_width + self.day_spacing)
                label = TransformedAndAlignedText((self.left_margin + (x - (width / 2) - (person_count_with_events / 2)), 
                                                   self.yUp(self.bottom_margin + ((self.end_hour * self.hour_height) + 4))),
                                                  "%s: %d/%d" % (self.event_container.getWeekdayAbbreviation(day.year, 
                                                                                                           day.month, 
                                                                                                           day.day),
                                                                 day.month,
                                                                 day.day),
                                                  rgb256(colors.black.rgb()),
                                                  14,
                                                  -90,
                                                  'start')
                var += label.strarray()

            day += day_delta
    
        # paint boundaries for days with events
        day_delta = timedelta(days = 1)
        item_width = 0
        day = self.first_day
        while day <= self.last_day:
            person_count_with_events = 0
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                if len(events_for_day) > 0:
                    person_count_with_events += 1
                    item_width += self.day_width + self.day_spacing
            day += day_delta
            if not person_count_with_events:
                item_width += self.day_width + self.day_spacing
            else:
                x_bg_rect = item_width - (person_count_with_events * (self.day_width + self.day_spacing)) - (person_count_with_events/2)
                y_bg_rect = self.start_hour * self.hour_height
                bg_rect_width = person_count_with_events * (self.day_width + self.day_spacing)
                bg_rect_height = (self.end_hour - self.start_hour) * self.hour_height

                line = Line((self.left_margin + x_bg_rect, 
                             self.yUp(self.bottom_margin + (y_bg_rect + bg_rect_height))),
                            (self.left_margin + x_bg_rect, 
                             self.yUp(self.bottom_margin + (y_bg_rect + bg_rect_height + 2))),
                            1.0,
                            rgb256(colors.black.rgb()))
                var += line.strarray()

                line = Line((self.left_margin + x_bg_rect + bg_rect_width, 
                             self.yUp(self.bottom_margin + (y_bg_rect + bg_rect_height))),
                            (self.left_margin + x_bg_rect + bg_rect_width, 
                             self.yUp(self.bottom_margin + (y_bg_rect + bg_rect_height + 2))),
                            1.0,
                            rgb256(colors.black.rgb()))
                var += line.strarray()

        # draw lines and labels for hours
        return var

    def getSVG(self):
        svg_code = self.getOpenerSection()
        #svg_code += self.getDefsSection()
        svg_code += self.getTeachingScheduleSection()
        #svg_code += self.getBottomPopupSection()
        svg_code += self.getCloserSection()
        return svg_code

    def getDrawing(self):
        '''Teaching Schedule Plot'''

        # make a preliminary pass to determine item_width
        day_delta = timedelta(days = 1)
        item_width = 0
        day = self.first_day
        while day <= self.last_day:
            person_count_with_events = 0
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                if len(events_for_day) > 0:
                    person_count_with_events += 1
                    item_width += self.day_width + self.day_spacing
            day += day_delta
            if not person_count_with_events:
                item_width += self.day_width + self.day_spacing 

        panel_width = item_width
        self.total_width = self.left_margin + panel_width + self.right_margin

        #panel_height = (self.end_hour - self.start_hour) * self.hour_height
        panel_height = 24 * self.hour_height
        self.total_height = self.bottom_margin + panel_height + self.top_margin
    
        D = Drawing(self.total_width, self.total_height) 
        g = Group(transform=(1,0,0,1,0,0))
    
        g.translate(self.left_margin, self.bottom_margin)
        #g.rotate(90)

        #tmp = self.total_height
        #self.total_height = self.total_width
        #self.total_width = tmp
 
        #tmp = panel_height
        #panel_height = panel_width
        #panel_width = tmp
    
        # paint background rects for days with events
        day_delta = timedelta(days = 1)
        item_width = 0
        day = self.first_day
        while day <= self.last_day:
            person_count_with_events = 0
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                if len(events_for_day) > 0:
                    person_count_with_events += 1
                    item_width += self.day_width + self.day_spacing
            day += day_delta
            if not person_count_with_events:
                item_width += self.day_width + self.day_spacing
            else:
                x_bg_rect = item_width - (person_count_with_events * (self.day_width + 2)) - (person_count_with_events/2)
                y_bg_rect = self.start_hour * self.hour_height
                bg_rect_width = person_count_with_events * (self.day_width + self.day_spacing)
                bg_rect_height = (self.end_hour - self.start_hour) * self.hour_height
                g.add(RLRect(x_bg_rect,
                             y_bg_rect,
                             bg_rect_width,
                             bg_rect_height,
                             fillColor=colors.lightgrey,
                             strokeColor=colors.black,
                             strokeWidth=0.5))

        # draw lines and labels for hours
        panel_width = item_width
        for hour in range(self.start_hour, self.end_hour+1):
            if hour == 12:
                strokeWidth = 1.0
            else:
                strokeWidth = 0.1
            g.add(RLLine(-2, hour * self.hour_height,
                         panel_width + 2, hour * self.hour_height,
                         strokeWidth = strokeWidth,
                         strokeColor = colors.black))
            label = Label()
            label.setOrigin(-4, hour * self.hour_height)
            label.angle = 0
            label.setText("%d:00" % hour)
            label.textAnchor = 'start'
            label.boxAnchor = 'e'
            label.fontName = 'Helvetica'
            label.fontSize = 14
            g.add(label)

            label = Label()
            label.setOrigin(panel_width + 4, hour * self.hour_height)
            label.angle = 0
            label.setText("%d:00" % hour)
            label.textAnchor = 'start'
            label.boxAnchor = 'w'
            label.fontName = 'Helvetica'
            label.fontSize = 14
            g.add(label)
                    
        # draw events
        day_delta = timedelta(days = 1)
        x = 0
        day = self.first_day
        while day <= self.last_day:
            person_count_with_events = 0
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                #if len(events_for_day) > 0:
                    #g.add(RLLine(x + ((self.day_width+self.day_spacing) / 2) - 1, 
                                 #self.start_hour * self.hour_height - 2, 
                                 #x + ((self.day_width+self.day_spacing) / 2) - 1, 
                                 #self.end_hour * self.hour_height,
                                 #strokeWidth = 0.1,
                                 #strokeDashArray = [1, 2],
                                 #strokeColor = colors.black))
                for event in events_for_day:
                    duration_hours = float(event.duration.seconds) / 3600.0
                    y = int((float(event.hour) + (float(event.minute) / 60.0)) * float(self.hour_height))
                    height = duration_hours * float(self.hour_height)
                    g.add(RLRect(x,
                                 y,
                                 self.day_width,
                                 height,
                                 fillColor=event.color,
                                 strokeWidth=0.5,
                                 strokeColor=colors.black))
                if len(events_for_day) > 0:
                    person_count_with_events += 1
    
                    label = Label()
                    label.setOrigin(x + (self.day_width/2) - 1,
                                    (self.start_hour * self.hour_height) - 4)
                    label.angle = 90
                    label.setText(person.username)
                    label.textAnchor = 'end'
                    label.boxAnchor = 'e'
                    label.fontName = 'Helvetica'
                    label.fontSize = 14
                    g.add(label)
                    x += self.day_width + self.day_spacing
    
            if not person_count_with_events:
                label = Label()
                label.setOrigin((x + (self.day_width / 2) + 1), 
                                (self.end_hour * self.hour_height) + 4)
                label.angle = 90
                label.setText("%s: %d/%d" % (self.event_container.getWeekdayAbbreviation(day.year, 
                                                                                       day.month, 
                                                                                       day.day),
                                             day.month,
                                             day.day))
                label.textAnchor = 'start'
                label.boxAnchor = 'w'
                label.fontName = 'Helvetica'
                label.fontSize = 14
                g.add(label)
                x += self.day_width + self.day_spacing
            else:
                width = person_count_with_events * (self.day_width + self.day_spacing)
                label = Label()
                label.setOrigin((x - (width / 2) - (person_count_with_events / 2)),
                                (self.end_hour * self.hour_height) + 4)
                label.angle = 90
                label.setText("%s: %d/%d" % (self.event_container.getWeekdayAbbreviation(day.year, 
                                                                                       day.month, 
                                                                                       day.day),
                                             day.month,
                                             day.day))
                label.textAnchor = 'start'
                label.boxAnchor = 'w'
                label.fontName = 'Helvetica'
                label.fontSize = 14
                g.add(label)
            day += day_delta

        # paint boundaries for days with events
        # do boundaries last to show them on top, if that matters
        day_delta = timedelta(days = 1)
        item_width = 0
        day = self.first_day
        while day <= self.last_day:
            person_count_with_events = 0
            for person in self.person_container.persons:
                events_for_day = self.event_container.getAllEventsForDateForPerson(day, person.username)
                if len(events_for_day) > 0:
                    person_count_with_events += 1
                    item_width += self.day_width + self.day_spacing
            day += day_delta
            if not person_count_with_events:
                item_width += self.day_width + self.day_spacing
            else:
                x_bg_rect = item_width - (person_count_with_events * (self.day_width + 2)) - (person_count_with_events/2)
                y_bg_rect = self.start_hour * self.hour_height
                bg_rect_width = person_count_with_events * (self.day_width + self.day_spacing)
                bg_rect_height = (self.end_hour - self.start_hour) * self.hour_height
                g.add(RLLine(x_bg_rect, 
                             y_bg_rect + bg_rect_height,
                             x_bg_rect, 
                             y_bg_rect + bg_rect_height + 2,
                             strokeWidth = 0.1,
                             strokeColor = colors.black))
                g.add(RLLine(x_bg_rect + bg_rect_width, 
                             y_bg_rect + bg_rect_height,
                             x_bg_rect + bg_rect_width, 
                             y_bg_rect + bg_rect_height + 2,
                             strokeWidth = 0.1,
                             strokeColor = colors.black))
    
        D.add(g)
        g = Group(transform=(1,0,0,1,0,0))
        return D

class Course:
    def __init__(self,x,y,radius,color):
        self.x = x
        self.y = y
        self.radius = radius
        self.color = color

class TeachingScheduleDrawing:
    def __init__(self, render_types, 
                 left_margin, right_margin, 
                 top_margin, bottom_margin, 
                 scale_to_fit, 
                 start_year, start_month, start_day, 
                 end_year, end_month, end_day, 
                 day_width, hour_height, bottom_hour, top_hour,
                 courses_input):
        self.render_types = render_types
        self.left_margin = left_margin
        self.right_margin = right_margin
        self.top_margin = top_margin
        self.bottom_margin = bottom_margin
        self.scale_to_fit = scale_to_fit
        self.start_year = start_year
        self.start_month = start_month
        self.start_day = start_day
        self.end_year = end_year
        self.end_month = end_month
        self.end_day = end_day
        self.day_width = day_width
        self.hour_height = hour_height
        self.bottom_hour = bottom_hour
        self.top_hour = top_hour
        self.courses_input = courses_input

        name = "teachingschedule"

        self.teaching_schedule = DayPlanningCalendar(self.left_margin, self.right_margin, self.top_margin, self.bottom_margin,
                                                     date(self.start_year, self.start_month, self.start_day),
                                                     date(self.end_year, self.end_month, self.end_day),
                                                     'faculty_schedule_test.html','faculty_schedule_test.pdf',
                                                     self.day_width, self.hour_height, self.bottom_hour, self.top_hour)
        teachers_and_courses = {}
        for course_string in self.courses_input:
            username, year, month, day, hour, minute, duration_minutes, recurrence_interval, start_year, start_month, start_day, end_year, end_month, end_day, course_name, red, green, blue  = eval('[%s]' % course_string)
            recurring_event = RecurringEvent(year, month, day, hour, minute, 
                                             timedelta(minutes=duration_minutes), 
                                             recurrence_interval, 
                                             date(start_year, start_month, start_day), 
                                             date(end_year, end_month, end_day),
                                             course_name, 
                                             getrlcolor(red, green, blue))
            if not teachers_and_courses.has_key(username):
                teachers_and_courses[username] = []
                teachers_and_courses[username].append(recurring_event)
            else:
                teachers_and_courses[username].append(recurring_event)
            self.teaching_schedule.addRecurringEvent(recurring_event)

        teachers = []
        for username in teachers_and_courses.keys():
            teachers.append(FacultyMember(username, '', 'Earth And Space Sciences', [], teachers_and_courses[username], colors.green))

        for teacher in teachers:
            for recurring_event in teachers_and_courses[teacher.username]:
                self.teaching_schedule.event_container.addAssociationForPersonToRecurringEvent(teacher.username, recurring_event)
            self.teaching_schedule.addPerson(teacher)

    def getDrawing(self):
        self.drawing = self.teaching_schedule.getDrawing()

        if self.scale_to_fit:
            scale_factor = float(self.height) / self.teaching_schedule.total_height
            self.drawing.scale(scale_factor,scale_factor)

        return self.drawing

    def getSVG(self):
        return self.teaching_schedule.getSVG()

    def getHTML(self):
        return self.teaching_schedule.getHTML()

    def getHTMLForContainingSVG(self, name):
        return self.teaching_schedule.getHTMLForContainingSVG(name)


